/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.action;

import java.awt.event.ActionEvent;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import edu.stanford.smi.protege.model.CommandManager;
import edu.stanford.smi.protege.model.DefaultKnowledgeBase;
import edu.stanford.smi.protege.model.KnowledgeBase;
import edu.stanford.smi.protege.model.framestore.undo.Command;
import edu.stanford.smi.protege.resource.ResourceKey;

/**
 * Request that the previous operation be undone.
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class UndoAction extends ProjectAction {

    public UndoAction(boolean large) {
        super(ResourceKey.UNDO_ACTION, large);
        CommandManager manager = getCommandManager();        
        if (manager == null || isUndoEnabled()) {
            setEnabled(false);
        } else {
            setEnabled(isUndoEnabled() & manager.canUndo());
            manager.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent event) {
                    setEnabled(isUndoEnabled() & getCommandManager().canUndo());
                }
            });
        }
    }

    private boolean isUndoEnabled() {
    	KnowledgeBase kb = getKnowledgeBase();    	
    	return (kb!=null && kb instanceof DefaultKnowledgeBase && ((DefaultKnowledgeBase)kb).isUndoEnabled() && kb.getProject().isUndoOptionEnabled());
	}

	public Object getValue(String key) {
        Object value = super.getValue(key);
        if (key.equals(NAME)) {
            // Log.stack("", this, "getValue", key);
            CommandManager commandManager = getCommandManager();
            if (commandManager != null) {
                Command command = commandManager.getUndoCommand();
                if (command != null) {
                    String s = command.getDescription();
                    if (s!= null && s.length() > 75) {
                        s = s.substring(0, 75) + "...";
                    }
                    value = value.toString() + " (" + s == null ? "no description" : s + ")";
                }
            }
        }
        return value;
    }

    public void actionPerformed(ActionEvent event) {
        getCommandManager().undo();
    }
}