/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.model.framestore.undo;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Facet;
import edu.stanford.smi.protege.model.Frame;
import edu.stanford.smi.protege.model.SimpleInstance;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protege.model.framestore.FrameStore;

public class DeleteFrameCommand extends SimpleCommand {
    private Frame frame;
    
    private Map<Slot, Collection> directSlotValues = new HashMap<Slot, Collection>();


    
    public DeleteFrameCommand(FrameStore delegate, Frame frame) {
        super(delegate);
        this.frame = frame;
    }
    
    public Frame getFrame() {
        return frame;
    }
    

    public Object doIt() {
        saveFrame();
        deleteFrame();
        frame.markDeleted(true);
        return null;
    }

    public void undoIt() {
        restoreFrame();
        frame.markDeleted(false);
    }
    
    @SuppressWarnings("unchecked")
    protected void saveFrame() {
        Set<Slot> slots = getDelegate().getOwnSlots(frame);
        for (Slot slot : slots) {
            Collection values = getDelegate().getDirectOwnSlotValues(frame, slot);
            if (values != null && !values.isEmpty()) {
                directSlotValues.put(slot, values);
            }
        }
    }
    
    @SuppressWarnings("unchecked")
    protected void restoreFrame() {
        for (Entry<Slot, Collection> entry : directSlotValues.entrySet()) {
            getDelegate().setDirectOwnSlotValues(frame, entry.getKey(), entry.getValue());
        }
    }
    
    private void deleteFrame() {
        if (frame instanceof Cls) {
            getDelegate().deleteCls((Cls) frame);
        }
        else if (frame instanceof Slot) {
            getDelegate().deleteSlot((Slot) frame);
        }
        else if (frame instanceof Facet) {
            getDelegate().deleteFacet((Facet) frame);
        }
        else {
            getDelegate().deleteSimpleInstance((SimpleInstance) frame);
        }
    }
    


}
