/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.plugin;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

class DefaultEntry {
    private boolean cardinalityMultiple;
    private String typeName;
    private String allowedClsName;
    private String slotname;

    DefaultEntry(String cardinality, String typeName, String allowedClsName, String slotname) {
        this.cardinalityMultiple = "multiple".equalsIgnoreCase(cardinality);
        this.typeName = typeName;
        this.allowedClsName = allowedClsName;
        this.slotname = slotname;
    }

    DefaultEntry(boolean cardinality, ValueType type, Cls allowedCls) {
        this.cardinalityMultiple = cardinality;
        this.typeName = type.toString();
        this.allowedClsName = (allowedCls == null) ? null : allowedCls.getName();
    }

    DefaultEntry(Slot slot) {
        this.slotname = slot.getName();
    }

    public boolean equals(Object o) {
        DefaultEntry rhs = (DefaultEntry) o;
        boolean isEquals = (cardinalityMultiple == rhs.cardinalityMultiple)
                && SystemUtilities.equals(typeName, rhs.typeName)
                && SystemUtilities.equals(allowedClsName, rhs.allowedClsName)
                && SystemUtilities.equals(slotname, rhs.slotname);
        return isEquals;
    }

    public int hashCode() {
        return HashUtils.getHash(typeName, allowedClsName, slotname, cardinalityMultiple);
    }

    public String toString() {
        return "DefaultEntry(" + cardinalityMultiple + ", " + typeName + ", " + allowedClsName + ")";
    }
}
