/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.server.metaproject.impl;

import java.util.Collection;
import java.util.Set;

import edu.stanford.smi.protege.exception.OntologyException;
import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.server.metaproject.Group;
import edu.stanford.smi.protege.server.metaproject.User;

public class GroupImpl extends PolicyControlledObjectImpl implements Group {
    private static final long serialVersionUID = -6623180400376787848L;

    protected GroupImpl(MetaProjectImpl mp, Instance group)
	throws OntologyException {
		super(mp, group, MetaProjectImpl.ClsEnum.Group);

	}

	@SuppressWarnings("unchecked")
	public Set<User> getMembers() {
		return getSlotValues(MetaProjectImpl.SlotEnum.member, MetaProjectImpl.ClsEnum.User);
	}


	public void addMember(User member) {
		addSlotValue(MetaProjectImpl.SlotEnum.member, member);
	}

	public void setMembers(Collection<User> members) {
		setSlotValuesAsProtegeInstances(MetaProjectImpl.SlotEnum.member, members);
	}

	@Override
    public String getDescription() {
		Object value = getProtegeInstance().getOwnSlotValue(getMetaProject().getSlot(MetaProjectImpl.SlotEnum.description));
		if (!(value instanceof String)) {
			throw new OntologyException("The " + MetaProjectImpl.SlotEnum.name + " slot should take on string values");
		}
		return (String) value;
	}
}
