/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.server.metaproject.impl;

import java.io.Serializable;
import java.util.Collection;
import java.util.Set;

import edu.stanford.smi.protege.exception.OntologyException;
import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.server.metaproject.GroupOperation;
import edu.stanford.smi.protege.server.metaproject.PolicyControlledObject;
import edu.stanford.smi.protege.server.metaproject.PropertyValue;

public class PolicyControlledObjectImpl extends WrappedProtegeInstanceImpl implements PolicyControlledObject,
        Serializable {
    private static final long serialVersionUID = -6929685166227007784L;

    private String name;

    protected PolicyControlledObjectImpl(MetaProjectImpl mp, Instance pi) {
        this(mp, pi, MetaProjectImpl.ClsEnum.PolicyControlledObject);
    }

    protected PolicyControlledObjectImpl(MetaProjectImpl mp, Instance pi, MetaProjectImpl.ClsEnum cls) {
        super(mp, pi, cls);
        name = (String) pi.getOwnSlotValue(mp.getSlot(MetaProjectImpl.SlotEnum.name));
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
        setSlotValue(MetaProjectImpl.SlotEnum.name, name);
    }

    public String getDescription() {
        Object value = getProtegeInstance().getOwnSlotValue(
                getMetaProject().getSlot(MetaProjectImpl.SlotEnum.description));
        if (value == null) {
            return "";
        }
        if (!(value instanceof String)) {
            throw new OntologyException("The " + MetaProjectImpl.SlotEnum.description
                    + " slot should take on string values");
        }
        return (String) value;
    }

    public void setDescription(String description) {
        setSlotValue(MetaProjectImpl.SlotEnum.description, description);
    }

    public void setAllowedGroupOperations(Collection<GroupOperation> groupOperations) {
        setSlotValuesAsProtegeInstances(MetaProjectImpl.SlotEnum.allowedGroupOperation, groupOperations);
    }

    public void addAllowedGroupOperations(GroupOperation groupOperation) {
        addSlotValue(MetaProjectImpl.SlotEnum.allowedGroupOperation, groupOperation);
    }

    @SuppressWarnings("unchecked")
    public Set<GroupOperation> getAllowedGroupOperations() {
        return getSlotValues(MetaProjectImpl.SlotEnum.allowedGroupOperation, MetaProjectImpl.ClsEnum.GroupOperation);
    }

	@SuppressWarnings("unchecked")
	public Set<PropertyValue> getPropertyValues() {
		return getSlotValues(MetaProjectImpl.SlotEnum.properties, MetaProjectImpl.ClsEnum.PropertyValue);
	}

	public void setPropertyValues(Collection<PropertyValue> propertyValues) {
		setSlotValuesAsProtegeInstances(MetaProjectImpl.SlotEnum.properties, propertyValues);		
	}
	
	 public void addPropertyValue(PropertyValue propertyValue) {
    	 addSlotValue(MetaProjectImpl.SlotEnum.properties, propertyValue);		
	}
    
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof PolicyControlledObject)) {
            return false;
        }
        PolicyControlledObject other = (PolicyControlledObject) o;
        return name.equals(other.getName());
    }

    @Override
    public int hashCode() {
        return name == null ? 41 : name.hashCode();
    }

    @Override
    public String toString() {
        return name;
    }
	
}
