/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.server.metaproject.impl;

import java.io.File;
import java.io.Serializable;

import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protege.model.ValueType;
import edu.stanford.smi.protege.server.metaproject.ProjectInstance;
import edu.stanford.smi.protege.server.metaproject.User;
import edu.stanford.smi.protege.util.CollectionUtilities;

public class ProjectInstanceImpl extends PolicyControlledObjectImpl implements ProjectInstance, Serializable {
	private static final long serialVersionUID = 8666270295698053695L;

	String location;

	@SuppressWarnings("unchecked")
	protected ProjectInstanceImpl(MetaProjectImpl mp, Instance pi) {
		super(mp, pi, MetaProjectImpl.ClsEnum.Project);
		if (pi != null) {
			location = (String) pi.getOwnSlotValue(mp.getSlot(MetaProjectImpl.SlotEnum.location));
		}
		localizeLocation(location);
	}

	public String getLocation() {
		return location;
	}

	private static String localizeLocation(String location) {
		if (location == null) {	return null; }		
		if (File.separatorChar != '\\') {
			location = location.replace('\\', File.separatorChar);
		}
		return location;
	}

	public ProjectInstance getAnnotationProject() {
		return (ProjectInstance) getSlotValue(MetaProjectImpl.SlotEnum.annotationProject, MetaProjectImpl.ClsEnum.Project);
	}

	public void setLocation(String location) {
		this.location = location;
		setSlotValue(MetaProjectImpl.SlotEnum.location, location);
	}


	public User getOwner() {
		return (User) getSlotValue(MetaProjectImpl.SlotEnum.owner, MetaProjectImpl.ClsEnum.User);
	}

	public void setAnnotationProject(ProjectInstance annotationProject) {		
		//make sure that the annotationProject slot is there
		Project prj = getProtegeInstance().getProject();
		Slot annotationProjectSlot = prj.getKnowledgeBase().getSlot(MetaProjectImpl.SlotEnum.annotationProject.name());
		if (annotationProjectSlot == null) {
			annotationProjectSlot = prj.getKnowledgeBase().createSlot(MetaProjectImpl.SlotEnum.annotationProject.name());
			annotationProjectSlot.setValueType(ValueType.INSTANCE);
			annotationProjectSlot.setAllowedClses(CollectionUtilities.createCollection(prj.getKnowledgeBase().getCls(MetaProjectImpl.ClsEnum.Project.name())));
		}
		
		setSlotValue(MetaProjectImpl.SlotEnum.annotationProject, annotationProject);
	}

	public void setOwner(User owner) {
		setSlotValue(MetaProjectImpl.SlotEnum.owner, owner);
	}

}
