/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.server.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

public class ZipSocket extends Socket {
  
  private InputStream in;
  private OutputStream out;
  
  public ZipSocket() {
    super();
  }
  
  public ZipSocket(String host, int port) throws UnknownHostException, IOException {
    super(host, port);
  }
  
  public InputStream getInputStream() throws IOException {
    if (in == null) {
      in = new ZipInputStream(super.getInputStream());
    }
    return in;
  }

  public OutputStream getOutputStream() throws IOException {
    if (out == null) {
      out = new ZipOutputStream(super.getOutputStream());
    }
    return out;
  }
  
  public synchronized void close() throws IOException {
    OutputStream o = getOutputStream();
    o.flush();
    super.close();
  }
}
