/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;

import javax.swing.JTree;


public abstract class TreeSelectionHelper<X> {

	private JTree tree;

	public TreeSelectionHelper(JTree tree) {
		this.tree = tree;
	}

	protected abstract Collection<X> getParents(X child);

	protected boolean isVisible(X element) {
		return true;
	}
	
	protected Collection<X> getRoots() {
		Collection<X> roots = new ArrayList<X>((Collection<X>) ((LazyTreeNode) tree.getModel().getRoot()).getUserObject());
		return roots;
	}

	public void setSelectedNode(X value) {
		WaitCursor cursor = new WaitCursor(tree);	        
		ArrayList<X> frames = new ArrayList<X>();
		getVisiblePathToRoot(value, frames);
		Collections.reverse(frames);
		ComponentUtilities.setSelectedObjectPath(tree, frames);
		cursor.hide();
	}

	
	protected Collection<X> filterHiddenRoots(Collection<X> roots) {
		Iterator<X> i = roots.iterator();
		while (i.hasNext()) {
			X root =  i.next();
			if (!isVisible(root)) {
				i.remove();
			}
		}
		return roots;
	}
	

	public void getVisiblePathToRoot(X elem, Collection<X> path) {
		Collection<X> roots = filterHiddenRoots(getRoots());	
		path.add(elem);
		if (!roots.contains(elem)) {
			boolean succeeded = getVisiblePathToRoot(elem, roots, path);
			if (!succeeded) {
				Log.getLogger().warning("No visible path found for " + elem);
			}
		}
	}

	private boolean getVisiblePathToRoot(X elem, Collection<X> roots, Collection<X> path) {
		boolean found = false;
		Iterator<X> i = getParents(elem).iterator();
		while (i.hasNext() && !found) {
			X parent = i.next();
			if (isVisible(parent) && !path.contains(parent)) {
				path.add(parent);
				if (roots.contains(parent)) {
					found = true;
				} else {
					found = getVisiblePathToRoot(parent, roots, path);
				}
				if (!found) {
					path.remove(parent);
				}
			}
		}
		return found;
	}

}
